/*
 * ConnectionTest.c
 *
 *  Created on: Nov 8, 2017
 *      Author: mangoh
 */


/**
 * This module is for testing of the modemServices MDC component.
 *
 * You must issue the following commands:
 * @verbatim
   $ app start mdcTest
   @endverbatim
 *
 * By default, the profile used is LE_MDC_DEFAULT_PROFILE, and the APN is automatically set.
 * Some customize parameters can be set by creating a "/tmp/config.txt" file and fill a command
 * line with the syntax:
 * <profile number> <pdp_type> <apn> [<authentification_type> <username> <password>]
 * Where:
 *  - <profile number> is the profile number to be used (or "default" to use the default profile)
 *  - <pdp_type> if the packet data protocol to be used: "ipv4", "ipv6", or "ipv4v6"
 *  - <apn> is the APN to be used
 *  - <authentification_type> (optional): authentification requested: "auth_none" (default), pap",
 *  "chap", "pap-chap"
 *  - <username> (optional): username for authentification
 *  - <password> (optional): password for authentification
 *
 * Copyright (C) Sierra Wireless Inc.
 *
 */

#include "legato.h"
#include "interfaces.h"
//#include "le_mdc_inteface.h"
#include "le_data_interface.h"
#include "le_print.h"
#include "errno.h"
//#include "WSServer.h"

#define TEST_DEF(X) {#X, X}

// Semaphore
static le_sem_Ref_t    TestSemaphore;
static le_sem_Ref_t    AsyncTestSemaphore;

//! [Profiles]
//static const char DefaultCid[] = "Default";
//static const char automaticApn[] = "Auto";
//static const char PdpIpv4[] = "ipv4";
//static const char PdpIpv6[] = "ipv6";
//static const char PdpIpv4v6[] = "ipv4v6";
//static const char AuthPap[] = "pap";
//static const char AuthChap[] = "chap";
//static const char AuthPapChap[] = "pap-chap";
//static const char Map[] = "map";
//static const char Cid[] = "cid";
//static const char Rmnet[] = "rmnet";
//static const char DefaultProfile[] = LE_MDC_DEFAULT_PROFILE;
//static const char VzwStatic[] = "we01.VZWSTATIC";
//static const char verizonDefault[] = "3";
//static le_data_RequestObjRef_t currentConnection;
//static char IMEI[50];
//static char ESN[50];
//static char fwVersion[50];
//static char MDN[20];
//static char PRI_ID[50];



// Structure used to set the configuration
typedef struct
{
    char   cid[10];                                 // profile identifier
    char   pdp[10];                                 // packet data protocol
    char   apn[LE_MDC_APN_NAME_MAX_LEN];            // access point network
    char   auth[10];                                // authentification level
    char   userName[LE_MDC_USER_NAME_MAX_LEN];      // username for authentification
    char   password[LE_MDC_PASSWORD_NAME_MAX_LEN];  // password for authentification
}
Configuration_t;

// Tests cases
typedef enum
{
    TEST_SYNC,
    TEST_ASYNC,
    TEST_MAX
} Testcase_t;

// Tests definition
struct
{
    char testName[20];
    Testcase_t  testCase;
} testsDef[] = {
                    TEST_DEF(TEST_SYNC),
                    TEST_DEF(TEST_ASYNC),
                    TEST_DEF(TEST_MAX)
};


int DoIPPassthrough(void);
#if 0

//--------------------------------------------------------------------------------------------------
/**
 * Session handler response for connection and disconnection.
 */
//--------------------------------------------------------------------------------------------------
static void SessionHandlerFunc
(
    le_mdc_ProfileRef_t profileRef,
    le_result_t result,
    void* contextPtr
)
{
    le_result_t* activationPtr = contextPtr;
    *activationPtr = result;

    LE_INFO("Session result %d for profile %d", result, le_mdc_GetProfileIndex(profileRef));

    le_sem_Post(AsyncTestSemaphore);
}


//--------------------------------------------------------------------------------------------------
/**
 * Start asynchronous session.
 */
//--------------------------------------------------------------------------------------------------
static void SessionStartAsync
(
    void* param1Ptr,
    void* param2Ptr
)
{
    le_mdc_ProfileRef_t profileRef = param1Ptr;

    le_mdc_StartSessionAsync(profileRef, SessionHandlerFunc, param2Ptr);
}

//--------------------------------------------------------------------------------------------------
/**
 * Stop asynchronous session.
 */
//--------------------------------------------------------------------------------------------------
static void SessionStopAsync
(
    void* param1Ptr,
    void* param2Ptr
)
{
    le_mdc_ProfileRef_t profileRef = param1Ptr;

    le_mdc_StopSessionAsync(profileRef, SessionHandlerFunc, param2Ptr);
}


//--------------------------------------------------------------------------------------------------
/**
 * Set the configuration.
 *
 */
//--------------------------------------------------------------------------------------------------
static void SetConfiguration
(
    le_mdc_ProfileRef_t* profileRefPtr
)
{
    // Check the configuration file
    FILE* configFilePtr = fopen("/tmp/config.txt","r");
    Configuration_t configuration;

    memset(&configuration, 0, sizeof(Configuration_t));

    // if configuration file absent, use default settings
    if (NULL == configFilePtr)
    {
        
        le_utf8_Copy(configuration.cid, verizonDefault, sizeof(configuration.cid), NULL);
        //le_utf8_Copy(configuration.cid, DefaultCid, sizeof(configuration.cid), NULL);
        le_utf8_Copy(configuration.pdp, PdpIpv4v6, sizeof(configuration.pdp), NULL);
        le_utf8_Copy(configuration.apn, automaticApn, sizeof(configuration.apn), NULL);
    }
    else
    {
        // Get the configuration from the file
        fseek(configFilePtr, 0, SEEK_END);
        uint32_t len = ftell(configFilePtr);
        char cmdLine[ len + 1 ];
        char* saveLinePtr;
        char* saveParamPtr;
        char* cmdLinePtr;

        memset(cmdLine,0,len+1);
        fseek(configFilePtr, 0, SEEK_SET);
        fread(cmdLine, 1, len, configFilePtr);
        fclose(configFilePtr);

        // Get first line
        cmdLinePtr = strtok_r(cmdLine, "\r\n", &saveLinePtr);

        // Get profile number
        char* cidPtr =  strtok_r(cmdLinePtr, " ", &saveParamPtr);
        LE_ASSERT(NULL != cidPtr);
        le_utf8_Copy(configuration.cid, cidPtr, sizeof(configuration.cid), NULL);

        // Get pdp type
        char* pdpPtr = strtok_r(NULL, " ", &saveParamPtr);
        LE_ASSERT(NULL != pdpPtr);
        le_utf8_Copy(configuration.pdp, pdpPtr, sizeof(configuration.pdp), NULL);

        // Get apn
        char* apnPtr = strtok_r(NULL, " ", &saveParamPtr);
        LE_ASSERT(NULL != apnPtr);
        le_utf8_Copy(configuration.apn, apnPtr, sizeof(configuration.apn), NULL);

        // Get authentification
        char* authPtr = strtok_r(NULL, " ", &saveParamPtr);
        if (NULL != authPtr)
        {
            le_utf8_Copy(configuration.auth, authPtr, sizeof(configuration.auth), NULL);
        }

        // Get username
        char* userNamePtr = strtok_r(NULL, " ", &saveParamPtr);
        if (NULL != userNamePtr)
        {
            le_utf8_Copy(configuration.userName, userNamePtr, sizeof(configuration.userName), NULL);
        }

        // Get password
        char* passwordPtr = strtok_r(NULL, " ", &saveParamPtr);
        if (NULL != passwordPtr)
        {
            le_utf8_Copy(configuration.password, passwordPtr, sizeof(configuration.password), NULL);
        }

        // Get optional lines
        cmdLinePtr = strtok_r(NULL, "\r\n", &saveLinePtr);

        while (cmdLinePtr)
        {
            char* optionPtr = strtok_r(cmdLinePtr, " ", &saveParamPtr);

            // Check mapping cid
            if (strncmp(optionPtr, Map, strlen(Map)) == 0)
            {
                LE_INFO("mapping cid");
                char* cidPtr = strtok_r(NULL, " ", &saveParamPtr);

                if (strncmp(cidPtr, Cid, strlen(Cid)) == 0)
                {
                    int cid = strtol((const char*) cidPtr+strlen(Cid), NULL, 10);

                    if (0 != errno)
                    {
                        LE_ERROR("Bad cid %d %m", errno);
                        exit(EXIT_FAILURE);
                    }

                    LE_INFO("map cid %d", cid);

                    char* rmnetPtr = strtok_r(NULL, " ", &saveParamPtr);

                    if (0 == strncmp(rmnetPtr, Rmnet, strlen(Rmnet)))
                    {
                        int rmnet = strtol(rmnetPtr+strlen(Rmnet), NULL, 10);

                        if (0 != errno)
                        {
                            LE_ERROR("Bad rmnet %d %m", errno);
                            exit(EXIT_FAILURE);
                        }

                        le_mdc_ProfileRef_t profileRef = le_mdc_GetProfile(cid);

                        char string[20];
                        memset(string,0,sizeof(string));
                        snprintf(string,sizeof(string),"rmnet_data%d", rmnet);
                        LE_INFO("cid %d rmnet: %s", cid, string);

                        LE_ASSERT(le_mdc_MapProfileOnNetworkInterface(profileRef, string) == LE_OK);
                    }
                }
            }

            cmdLinePtr = strtok_r(NULL, "\r\n", &saveLinePtr);
        }
    }

    uint32_t profile;

    if ( strncmp(configuration.cid, DefaultCid, sizeof(DefaultCid)) == 0 )
    {
        profile = LE_MDC_DEFAULT_PROFILE;
    }
    else
    {
        errno = 0;
        profile = strtol(configuration.cid, NULL, 10);

        if (errno != 0)
        {
            LE_ERROR("Bad profile %d %m", errno);
            exit(EXIT_FAILURE);
        }
    }

    // Get the profile reference
    *profileRefPtr = le_mdc_GetProfile(profile);
    LE_ASSERT(NULL != *profileRefPtr);

    // Check the current state of the cid
    le_mdc_ConState_t state = LE_MDC_DISCONNECTED;

    // Check the state
    LE_ASSERT(LE_OK == le_mdc_GetSessionState(*profileRefPtr, &state));

    // If already connected, disconnect the session
    if (LE_MDC_CONNECTED == state)
    {
        LE_ASSERT(le_mdc_StopSession(*profileRefPtr) == LE_OK);
    }

    // Set pdp type
    le_mdc_Pdp_t pdp = LE_MDC_PDP_UNKNOWN;

    if (0 == strncmp(configuration.pdp, PdpIpv4, sizeof(PdpIpv4)))
    {
        pdp = LE_MDC_PDP_IPV4;
    }
    else if (0 == strncmp(configuration.pdp, PdpIpv6, sizeof(PdpIpv6)))
    {
        pdp = LE_MDC_PDP_IPV6;
    }
    else if (0 == strncmp(configuration.pdp, PdpIpv4v6, sizeof(PdpIpv4v6)))
    {
        pdp = LE_MDC_PDP_IPV4V6;
    }

    LE_ASSERT(LE_OK == le_mdc_SetPDP(*profileRefPtr, pdp));

#if 0
    // Set APN
    if (0 == strncmp(configuration.apn, automaticApn, sizeof(automaticApn)))
    {
        // Set default APN
        LE_ASSERT(LE_OK == le_mdc_SetDefaultAPN(*profileRefPtr));
    }
    else
    {
        LE_ASSERT(LE_OK == le_mdc_SetAPN(*profileRefPtr, configuration.apn));
    }
#endif
    LE_ASSERT(LE_OK == le_mdc_SetAPN(*profileRefPtr, "we01.VZWSTATIC"));
    le_mdc_Auth_t auth = LE_MDC_AUTH_NONE;
    if ('\0' != configuration.auth[0])
    {
        // Set the authentification, username and password
        if (0 == strncmp(configuration.auth, AuthPapChap, sizeof(AuthPapChap)))
        {
            auth = LE_MDC_AUTH_PAP | LE_MDC_AUTH_CHAP;
        }
        // Set the authentification, username and password
        else if (0 == strncmp(configuration.auth, AuthPap, sizeof(AuthPap)))
        {
            auth = LE_MDC_AUTH_PAP;
        }
        else if (0 == strncmp(configuration.auth, AuthChap, sizeof(AuthChap)))
        {
            auth = LE_MDC_AUTH_CHAP;
        }

        if (LE_MDC_AUTH_NONE != auth)
        {
            LE_ASSERT(LE_OK == le_mdc_SetAuthentication(*profileRefPtr,
                                                auth,
                                                configuration.userName,
                                                configuration.password));
        }
    }

    LE_INFO("cid: %d pdp: %d apn: %s auth: %d username: %s password: %s", profile, pdp,
            configuration.apn, auth, configuration.userName, configuration.password);
}
#endif
void DataConnectionHandler(bool isConnected, void * contextPtr)
{
	bool * connect = (bool *)(contextPtr);

    *connect = isConnected;
}

static int GetNextAPN(char * currentAPN)
{
    int success = -1;
    bool next = false;
    FILE* apnFile = NULL;

    static char apn1[15];
    static char apn2[15];
    static char apn3[15];
    static char apn4[15];
    static char apn5[15];
    static char apn6[15];
    static char apn7[15];
    static char apn8[15];
    static char apn9[15];
    static char apn10[15];
    char * apns[10] = {apn1,apn2,apn3,apn4,apn5,apn6,apn7,apn8,apn9,apn10};
    
    if (currentAPN == NULL)
    {
        success = -1;
    }
    else
    { 
        LE_INFO("GetNextAPN recieved currentAPN = %s",currentAPN );
        if (currentAPN[0] != 0)
        {
            next = true;
        }
        apnFile = fopen("/home/root/311480","r");

        if (apnFile != NULL)
        {
            bool keepGoing = true;
            int apnNumber = 0, index;
            char byte = 1, numRead = 0;

            while ((byte != 0) && (apnNumber < 10))
            {
                index = 0;
                keepGoing = true;
                while (keepGoing)
                {
                    unsigned char byte;
                    numRead = fread(&byte,1,1,apnFile);
                    if ((numRead == 0) || (byte == 0))
                    {
                        //LE_INFO("Retrieved APN %d, %s",apnNumber, apns[apnNumber]);
                        keepGoing = false;
                        apns[apnNumber][index] = 0;
                        //LE_INFO("numRead = %d, byte = %d",numRead, byte);
                        apnNumber++;
                        byte = 0;
                        break;
                    }
                    else if (byte == 0x0D)
                    {
                        fread(&byte,1,1,apnFile);
                        if (byte != 0x0A)
                            LE_INFO("byte read shoud have been 0x0A");
                        keepGoing = false;
                        apns[apnNumber][index] = 0;
                        LE_INFO("Retrieved APN %d, %s",apnNumber, apns[apnNumber]);
                        apnNumber++;
                    }
                    else
                        apns[apnNumber][index++] = byte;

                }
            }
            if (next)
            {
                // TO DO  - Needs to compare the currentAPN with each APN in the list and see if it is it.  
                // If it is, return the next one.  If it exceeds our apn list, then return 0.
                int apnIndex = 0;
                while ((strncmp(currentAPN, apns[apnIndex],15) != 0) && (apnIndex < 10))
                {
                    apnIndex++;
                	//LE_INFO("apnIndex = %d",apnIndex);
                }
                if(apnIndex < 9)
                	apnIndex++;
                strncpy(currentAPN, apns[apnIndex],15);
                LE_INFO("GetNextAPN returning %s",currentAPN);
                success = LE_OK;
            }
            else
            {
                strncpy(currentAPN, apns[0],15);
                success = LE_OK;
            }
        }
        else
        {
            LE_ERROR("Couldn't open APN file!");
        }
    }

    return (success);
}
//! [Profiles]

//! [Sessions]
//--------------------------------------------------------------------------------------------------
/**
 * Set the network configuration.
 *
 */
//--------------------------------------------------------------------------------------------------
static void SetNetworkConfiguration
(
    le_mdc_ProfileRef_t profileRef
)
{
    char ipAddr[100] = {0};
    char gatewayAddr[100] = {0};
    char dns1Addr[100] = {0};
    char dns2Addr[100] = {0};
    char systemCmd[200] = {0};
    FILE* resolvFilePtr;
    le_mdc_ConState_t state = LE_MDC_DISCONNECTED;
    mode_t oldMask;
    int result = LE_OK;

    // Check the state
    if ( (result = le_mdc_GetSessionState(profileRef, &state)) != LE_OK )
        LE_INFO("le_mdc_GetSessionState returned %d", result);
    if ( state != LE_MDC_CONNECTED )
    {
        LE_INFO("SetNetworkConfiguration - not connected!");
        return;
    }

    // Get IP, gateway and DNS addresses for IPv4 or IPv6 connectivity
    if ( le_mdc_IsIPv4(profileRef) )
    {
        LE_INFO("Setting IPV4 routing");
        if (le_mdc_GetIPv4Address(profileRef, ipAddr, sizeof(ipAddr)) != LE_OK)
            LE_INFO("Can't get IP address!");
        LE_INFO("IP Address: %s", ipAddr);

        if (le_mdc_GetIPv4GatewayAddress(profileRef, gatewayAddr, sizeof(gatewayAddr)) != LE_OK)
            LE_INFO("Can't get the gateway Address");
        LE_INFO("Gateway: %s", gatewayAddr);

        if (le_mdc_GetIPv4DNSAddresses( profileRef,
                                              dns1Addr, sizeof(dns1Addr),
                                              dns2Addr, sizeof(dns2Addr)) != LE_OK )
            LE_INFO("Can't get DNS addresses!");
        LE_INFO("DNS1: %s", dns1Addr);
        LE_INFO("DNS2: %s", dns2Addr);

        snprintf(systemCmd, sizeof(systemCmd), "/sbin/route add default gw %s", gatewayAddr);
        sleep(15);

        LE_DEBUG("%s", systemCmd);

        result = le_data_AddRoute(gatewayAddr);

        //result = system(systemCmd);
    }
    else // Check both?
        if ( le_mdc_IsIPv6(profileRef) )
    {
        LE_INFO("Setting IPV6 routing");
        if(le_mdc_GetIPv6Address(profileRef, ipAddr, sizeof(ipAddr)) != LE_OK)
            LE_INFO("CAn't get IPV6 address!");
        LE_PRINT_VALUE("%s", ipAddr);

        if (le_mdc_GetIPv6GatewayAddress(profileRef, gatewayAddr, sizeof(gatewayAddr)) != LE_OK)
            LE_INFO("Can't get IPV6 Gateway address!");
        LE_PRINT_VALUE("%s", gatewayAddr);

        if (le_mdc_GetIPv6DNSAddresses( profileRef,
                                              dns1Addr, sizeof(dns1Addr),
                                              dns2Addr, sizeof(dns2Addr)) != LE_OK )
        {
            LE_INFO("Can't get IPV6 DNS addresses!");
        }
        LE_PRINT_VALUE("%s", dns1Addr);
        LE_PRINT_VALUE("%s", dns2Addr);

        snprintf(systemCmd, sizeof(systemCmd), "/sbin/route -A inet6 add default gw %s",
                                                gatewayAddr);
        sleep(15);

        LE_DEBUG("%s", systemCmd);

        result = system(systemCmd);

    }

   
    // Start the app.
    //le_result_t result = le_appCtrl_Start(systemCmd);
    if (result != LE_OK)
    {
        //fprintf(stderr, "App '%s' could not be started. Check logs for more info.\n", systemCmd);
        LE_ERROR("cmd '%s' did returned %d. Check logs for more info.\n",systemCmd,result);
        //exit(EXIT_FAILURE);
    }


    // allow fopen to create file with mode=644
    oldMask = umask(022);

    // open the resolver configuration
    resolvFilePtr = fopen("/etc/resolv.conf", "w+");

    if (resolvFilePtr == NULL)
    {
        LE_ERROR("Unable to open resolv.conf: %m");
    }
    //LE_INFO( resolvFilePtr != NULL );

    fprintf(resolvFilePtr, "nameserver %s\n", dns1Addr);

    if (dns2Addr[0] != '\0')
    {
        fprintf(resolvFilePtr, "nameserver %s\n", dns2Addr) ;
    }

    fclose(resolvFilePtr);

    // restore old mask
    umask(oldMask);
}
//! [Sessions]

//--------------------------------------------------------------------------------------------------
/**
 * Handler function for session state Notifications.
 *
 */
//--------------------------------------------------------------------------------------------------
static void StateChangeHandler
(
    le_mdc_ProfileRef_t profileRef,
    le_mdc_ConState_t ConnectionStatus,
    void* contextPtr
)
{
    LE_DEBUG("\n================================================");
    LE_PRINT_VALUE("%d", (int) le_mdc_GetProfileIndex(profileRef));
    LE_PRINT_VALUE("%u", ConnectionStatus);

    if (ConnectionStatus == LE_MDC_DISCONNECTED)
    {
    	LE_INFO("--Modem Disconnected--");
        // Get disconnection reason
        LE_INFO("%d", le_mdc_GetDisconnectionReason(profileRef));
        LE_INFO("%d", le_mdc_GetPlatformSpecificDisconnectionCode(profileRef));
    }
    else if (ConnectionStatus == LE_MDC_CONNECTED)
	{
		LE_INFO("--Connected--");
		//sleep(5);
		DoIPPassthrough();
	}
	else
		LE_INFO("ConnectionStatus = %d", (int)ConnectionStatus);

    LE_DEBUG("\n================================================");

    // Post a semaphore to synchronize the test
    le_sem_Post(TestSemaphore);
}

//--------------------------------------------------------------------------------------------------
/**
 * Test thread (to run the event loop and calling the event handler).
 *
 */
//--------------------------------------------------------------------------------------------------
static void* ConnectionStateThread
(
    void* contextPtr
)
{
    le_mdc_ConnectService();

    le_mdc_ProfileRef_t profileRef = *((le_mdc_ProfileRef_t*)contextPtr);

    // Add state handler on the profile
    le_mdc_AddSessionStateHandler(profileRef, StateChangeHandler, NULL);

    // Post a semaphore to synchronize the test
    le_sem_Post(TestSemaphore);

    // Run the event loop
    LE_INFO("ConnMgr - Run Event Loop...");
    le_event_RunLoop();
}



//! [Statistics]
//--------------------------------------------------------------------------------------------------
/**
 * Test the connectivity.
 *
 */
//--------------------------------------------------------------------------------------------------
int TestConnectivity
(
    le_mdc_ProfileRef_t profileRef
)
{
    int result = LE_OK;
	LE_INFO("Testing Connectivity...");
    le_result_t res;
    char systemCmd[200] = {0};
    char itfName[LE_MDC_INTERFACE_NAME_MAX_BYTES]="\0";

    char * pingURL = "google.com";
    char * altPingURL = "bing.com";

    le_mdc_DataBearerTechnology_t downlinkDataBearerTech;
    le_mdc_DataBearerTechnology_t uplinkDataBearerTech;

    res = le_mdc_GetDataBearerTechnology(profileRef,
                                             &downlinkDataBearerTech,
                                             &uplinkDataBearerTech) ;
    if (res != LE_OK)
    {   
       LE_INFO("No WAN!");
       return(-2); //
    }

    LE_INFO("downlinkDataBearerTech %d, uplinkDataBearerTech %d",
                downlinkDataBearerTech, uplinkDataBearerTech);

    // Get interface name

    res = le_mdc_GetInterfaceName(profileRef, itfName, LE_MDC_INTERFACE_NAME_MAX_BYTES);
    if (res != LE_OK)
    {   
       LE_INFO("No WAN interface!");
       return(-3); //
    }


    if ( le_mdc_IsIPv4(profileRef) )
    {
        snprintf(systemCmd, sizeof(systemCmd), "ping -c 4 %s -I %s", pingURL, itfName);
    }
    else
    {
        // TODO ping6 needs raw access to socket and therefore root permissions => find a different
        // way to test the connectivity
        snprintf(systemCmd, sizeof(systemCmd), "ping6 -c 4 %s -I %s", pingURL, itfName);
    }

    res = system(systemCmd);
    if (res != LE_OK)
    {
    	LE_INFO("Couldn't ping %s with %s", pingURL, itfName);

        if ( le_mdc_IsIPv4(profileRef) )
        {
            snprintf(systemCmd, sizeof(systemCmd), "ping -c 4 %s -I %s", altPingURL, itfName);
        }
        else
        {
            // TODO ping6 needs raw access to socket and therefore root permissions => find a different
            // way to test the connectivity
            snprintf(systemCmd, sizeof(systemCmd), "ping6 -c 4 %s -I %s", altPingURL, itfName);
        }
        res = system(systemCmd);
    }
    if (result != -1)
    {
        // Get data counters
        uint64_t rxBytes=0, txBytes=0;
        le_mdc_GetBytesCounters(&rxBytes, &txBytes);

        LE_INFO("rxBytes %ld, txBytes %ld", (long int) rxBytes, (long int) txBytes);
    }
    else
        LE_INFO("Couldn't ping %s with %s", altPingURL, itfName);
    return (result);
}

void WaitForRegistration(void)
{
    le_result_t           res;
    le_mrc_NetRegState_t  regState;
    res = le_mrc_GetNetRegState(&regState);

    while (regState != LE_MRC_REG_HOME && regState != LE_MRC_REG_ROAMING)
    {
        // Not registered on the network yet.  Wait.
        LE_INFO("Modem not registered.");
        sleep(5);
        res = le_mrc_GetNetRegState(&regState);
        if (res != LE_OK)
        	LE_INFO("GetRegState call failed.");
    }

}

int DoIPPassthrough(void)
{
	int res = 0;
	// setup IP Passthrough
	LE_INFO("Calling IP Passthrough script.");
	char systemCmd[200] = {0};
	snprintf(systemCmd, sizeof(systemCmd), "/bin/sh /home/root/ippassthrough.sh > /home/root/ippassOutput.txt 2>&1");

	sleep(10);

	res = system(systemCmd);
	LE_INFO("ippassthrough returned %d",res);
	return (res);
}

void GetSettings(void)
{
	char settings[255] = {0};
	LE_INFO("GetSettings, called");
	//WSServer_GetConnectionSettings(settings);
	LE_INFO("Got Settings, %s", settings);
}


//! [Statistics]
COMPONENT_INIT
{
    sleep(10);
    le_mdc_ProfileRef_t profileRef  = NULL;
    le_clk_Time_t myTimeout = { 0, 0 };
    myTimeout.sec = 120;
    le_result_t res;
    //Testcase_t test = TEST_SYNC;
    le_thread_Ref_t connStateThread;
    bool firstTime = false, needRoute;


    char currentAPN[20] = {0};
    char defaultAPN[20] = {0};
    bool connected = false, failed = false;
    int pingInterval = 2*60;
    int pingRetries = 5;
    int pingRetryInterval = 5;


    int32_t currentProfileIndex = le_data_GetCellularProfileIndex();

    TestSemaphore = le_sem_Create("TestSemaphore",0);
    AsyncTestSemaphore = le_sem_Create("AsyncTestSemaphore",0);


    LE_INFO("======= Connection Manager started =======");
    needRoute = !le_data_GetDefaultRouteStatus();

    if(currentProfileIndex !=3) //Verizon default is always 3.
    {
    	currentProfileIndex = 3;
    	le_data_SetCellularProfileIndex(currentProfileIndex);
    }
    // Get the profile reference
    profileRef = le_mdc_GetProfile(currentProfileIndex);
    // Check the current state of the cid
    le_mdc_ConState_t state = LE_MDC_DISCONNECTED;

    // Check the state
    le_mdc_GetSessionState(profileRef, &state);

    // If already connected, disconnect the session
    if (LE_MDC_CONNECTED == state)
    {
        le_mdc_StopSession(profileRef);
    }

    connStateThread = le_thread_Create("Connect Thread", ConnectionStateThread, &profileRef);

     // Start a thread to treat the event handler.
    LE_INFO("ConnMgr: Starting Event Handler thread.");
    le_thread_Start(connStateThread);

    // Wait for the call of the event handler
    res = le_sem_WaitWithTimeOut(TestSemaphore, myTimeout);
    
    //res = le_mdc_MapProfileOnNetworkInterface(profileRef,"ecm0");
    char * resS = "LE_OK";

    if(res == LE_UNSUPPORTED)
    	resS = "LE_UNSUPPORTED";
    else if (res == LE_FAULT)
    	resS = "LE_FAULT";
    LE_INFO("premapping returned %s", resS);

    LE_INFO("ConnMgr: Wait for registration.");

    WaitForRegistration();

    LE_INFO("Starting Session with defaults.");

    le_data_RequestObjRef_t dataSession = NULL;
    dataSession = le_data_Request();

    if (dataSession == NULL)
    	LE_INFO("No DataSession!");

    le_mdc_GetSessionState(profileRef, &state);

    connected = state == LE_MDC_CONNECTED;


    //connected = le_mdc_StartSession(profileRef) == LE_OK;

    if (!connected)
    {
    	LE_INFO("First connection attempt didn't work");
    	//With Verizon, we always connect with both types.
    	le_mdc_Pdp_t pdp = LE_MDC_PDP_IPV4V6;
    	le_mdc_SetPDP(profileRef, pdp);
    }
    sleep(2);
    while (!connected || !firstTime)
    {
        WaitForRegistration();
    	if (!firstTime)
    	{
    	    LE_INFO("ConnMgr: First Time.");
    		firstTime = true;
    	}
        while (!connected && !failed)
        {
            if ((currentAPN[0] == 0) && (defaultAPN[0] != 0))
            {
                strncpy(currentAPN, defaultAPN, sizeof(currentAPN));
                failed = 0;
            }
            else
                failed = GetNextAPN(currentAPN) != LE_OK;
            LE_INFO("GetNextAPN returned %d, CurrentAPN = %s", failed,currentAPN);
            if (currentAPN[0] == 0 )
            	failed = GetNextAPN(currentAPN) != LE_OK; // start over
            le_mdc_SetAPN(profileRef, currentAPN);

            dataSession = le_data_Request();

            le_mdc_GetSessionState(profileRef, &state);

            connected = state == LE_MDC_CONNECTED;
        }
        if (!connected)
        {
            //Wait for the modem to register.
            sleep(5);  
            continue;
        }
        else
            strncpy( defaultAPN, currentAPN,sizeof(defaultAPN));
        if((res = le_mdc_ResetBytesCounter()) != LE_OK )
            LE_INFO("le_mdc_ResetBytesCounter returned %d", (int)res);


        LE_INFO("calling GetSettings");
        GetSettings();
        LE_INFO("called GetSettings");
        // Wait for the call of the event handler
        res = le_sem_WaitWithTimeOut(TestSemaphore, myTimeout);


        LE_INFO("Setting up the Network.");
        if (needRoute )
        {
        	// Set the network configuration
        	SetNetworkConfiguration(profileRef);
        }

        LE_INFO("Network setup.  Waiting");

        sleep(5);

        le_mdc_GetSessionState(profileRef, &state);

        while (state == LE_MDC_CONNECTED)
        {
            GetSettings();
            le_clk_Time_t pingTimeout = { pingInterval, 0 };
            res = le_sem_WaitWithTimeOut(TestSemaphore, pingTimeout);
            //sleep(300);

            
            int i;
            for (i = 0; i<pingRetries; i++)
            {
               // Test the new interface
                LE_INFO("Testing Connectivity");
                if(TestConnectivity(profileRef) == -1)
                    sleep(pingRetryInterval);
                else
                    break;
            }
            le_mdc_GetSessionState(profileRef, &state);

        }
        connected = false;
        failed = false;
        currentAPN[0] = 0;
        LE_ERROR("Retrying connection");
    } //while (!connected || !firstTime)

    LE_ASSERT(le_mdc_StopSession(profileRef) == LE_OK);
    le_thread_Cancel(connStateThread);

    LE_INFO("\n\n-------------ConnAPP is exiting--------------\n\n");
    exit(EXIT_SUCCESS);
}
